@@ -1,7 +1,15 @@
+# -*- coding: utf-8 -*-
+"""
+# --------------------------------------------------------
+# @Author : Pan
+# @E-mail :
+# @Date   : 2025-02-18 16:10:49
+# @Brief  :
+# --------------------------------------------------------
+"""
 import cv2
 import matplotlib.pyplot as plt
 import numpy as np
-import torch
 
 VOC_NAMES = ["aeroplane", "bicycle", "bird", "boat", "bottle", "bus", "car", "cat",
              "chair", "cow", "diningtable", "dog", "horse", "motorbike", "person",
@@ -20,6 +28,7 @@ COCO_NAMES = ['background', 'person', 'bicycle', 'car', 'motorcycle', 'airplane'
               'oven', 'toaster', 'sink', 'refrigerator', 'book', 'clock', 'vase', 'scissors',
               'teddy bear', 'hair drier', 'toothbrush']
 
+
 def get_colormap(data_type="custom"):
     if data_type == 'pascal':
         num_classes = 21
@@ -38,15 +47,6 @@ def get_colormap(data_type="custom"):
     return colormap, num_classes
 
 
-def decode_seg_map_sequence(label_masks, data_type='custom'):
-    rgb_masks = []
-    for label_mask in label_masks:
-        rgb_mask = decode_segmap(label_mask, data_type)
-        rgb_masks.append(rgb_mask)
-    rgb_masks = torch.from_numpy(np.array(rgb_masks).transpose([0, 3, 1, 2]))
-    return rgb_masks
-
-
 def decode_segmap(label_mask, data_type="custom", plot=False):
     """Decode segmentation class labels into a color image
     Args:
@@ -98,13 +98,14 @@ def decode_color_mask(mask, data_type="custom", plot=False) -> np.ndarray:
     return omask
 
 
-def decode_color_image_mask(image, mask, data_type='custom'):
+def draw_image_mask_color(image, mask, data_type='custom'):
     """
     :param image: BGR原始图像
     :param mask: 对应的Mask
     :param data_type: 数据类型
     :return:
     """
+    if mask.max() >= 255: mask = np.array(mask / 255, dtype=np.uint8)
     color_mask = decode_color_mask(mask, data_type=data_type, plot=False)
     alpha = mask.copy()
     alpha[mask > 0] = 255
@@ -118,6 +119,9 @@ def decode_color_image_mask(image, mask, data_type='custom'):
     return color_image, color_mask
 
 
+decode_color_image_mask = draw_image_mask_color
+
+
 def encode_segmap(mask):
     """Encode segmentation label images as pascal classes
     Args:
