@@ -1,13 +1,14 @@
 # -*- coding: utf-8 -*-
 """
 # --------------------------------------------------------
-# @Author : panjq
-# @E-mail : pan_jinquan@163.com
+# @Author : Pan
+# @E-mail : 390737991@qq.com
 # @Date   : 2019-12-31 09:11:25
 # --------------------------------------------------------
 """
 import os
 import time
+import re
 import shutil
 import numpy as np
 import json
@@ -17,24 +18,35 @@ import subprocess
 import concurrent.futures
 import numbers
 import pickle
+import argparse
+import itertools
 from datetime import datetime
+from tqdm import tqdm
+from pybaseutils import text_utils
 
-IMG_POSTFIX = ['*.jpg', '*.jpeg', '*.png', '*.tif']
+IMG_POSTFIX = ['*.jpg', '*.jpeg', '*.png', '*.tif', "*.JPG", "*.bmp"]
+VIDEO_POSTFIX = ['*.mp4', '*.avi', '*.mov', "*.flv", "*.dav"]
+AUDIO_POSTFIX = ['*.mp3', '*.wav']
 
 
 def str2bool(v):
-    return v.lower() in ('yes', 'true', 't', 'y', '1')
+    if v.lower() in ('yes', 'true', 't', 'y', '1'):
+        return True
+    elif v.lower() in ('no', 'false', 'f', 'n', '0'):
+        return False
+    else:
+        raise argparse.ArgumentTypeError('Unsupported value encountered.')
 
 
-def get_time(format="S"):
+def get_time(format="p"):
     """
     :param format:
     :return:
     """
-    if format in ["S", "s"]:
+    if format.lower() == "s":
         # time = datetime.strftime(datetime.now(), '%Y%m%d_%H%M%S')
         time = datetime.strftime(datetime.now(), '%Y%m%d%H%M%S')
-    elif format in ["P", "p"]:
+    elif format.lower() == "p":
         # 20200508_143059_379116
         time = datetime.strftime(datetime.now(), '%Y%m%d_%H%M%S_%f')
         time = time[:-2]
@@ -56,6 +68,26 @@ def get_kwargs_name(**kwargs):
     return prefix
 
 
+def get_file_size(file, rate=1024):
+    """获得文件大小"""
+    return os.path.getsize(file) / rate
+
+
+def replace_elements(items, src, dst, ignore=True):
+    """
+    将列表中，值为src改为dst
+    :param items:
+    :param src:
+    :param dst:
+    :param ignore: 是否提出空的数据
+    :return:
+    """
+    for i, item in enumerate(items):
+        if item == src: items[i] = dst
+    if ignore: items = [it for it in items if str(it)]
+    return items
+
+
 def combine_flags(flags: list, use_time=True, info=True):
     """
     :param flags:
@@ -76,6 +108,17 @@ def combine_flags(flags: list, use_time=True, info=True):
     return out_flags
 
 
+def write_file(file, data):
+    """写二进制数据"""
+    with open(file, 'wb') as f: f.write(data)
+
+
+def read_file(file):
+    """读取二进制数据"""
+    with open(file, 'rb') as f: key = f.read()
+    return key
+
+
 class WriterTXT(object):
     """ write data in txt files"""
 
@@ -84,18 +127,15 @@ class WriterTXT(object):
         if filename:
             self.f = open(filename, mode=mode)
 
-    def write_line_str(self, line_str, endline="\n"):
+    def write_line(self, line, end='\n'):
         if self.f:
-            line_str = line_str + endline
-            self.f.write(line_str)
+            self.f.write(line + end)
             self.f.flush()
 
-    def write_line_list(self, line_list, endline="\n"):
+    def write_line_list(self, line_list, end='\n'):
         if self.f:
-            for line_list in line_list:
-                # 将list转为string
-                line_str = " ".join('%s' % id for id in line_list)
-                self.write_line_str(line_str, endline=endline)
+            for line in line_list:
+                self.write_line(line, end=end)
             self.f.flush()
 
     def close(self):
@@ -128,23 +168,23 @@ def read_json_data(json_path):
     :param json_path:
     :return:
     """
-    with open(json_path, 'r') as f:
+    with open(json_path, 'rb') as f:
         json_data = json.load(f)
     return json_data
 
 
-def write_json_path(out_json_path, json_data):
+def write_json_path(json_file, json_data):
     """
     写入 JSON 数据
-    :param out_json_path:
+    :param json_file:
     :param json_data:
     :return:
     """
-    with open(out_json_path, 'w', encoding="utf-8") as f:
+    with open(json_file, 'w', encoding="utf-8") as f:
         json.dump(json_data, f, indent=4, ensure_ascii=False)
 
 
-def write_data(filename, content_list, split=" ", mode='w'):
+def write_data(filename, content_list, split=",", mode='w'):
     """保存list[list[]]的数据到txt文件
     :param filename:文件名
     :param content_list:需要保存的数据,type->list
@@ -173,7 +213,7 @@ def write_list_data(filename, list_data, mode='w'):
         f.flush()
 
 
-def read_data(filename, split=" ", convertNum=True):
+def read_data(filename, split=",", convertNum=True):
     """
     读取txt数据函数
     :param filename:文件名
@@ -187,32 +227,47 @@ def read_data(filename, split=" ", convertNum=True):
     注意：这些函数都只会删除头和尾的字符，中间的不会删除。
     """
     with open(filename, mode="r", encoding='utf-8') as f:
-        content_list = f.readlines()
-        if split is None:
-            content_list = [content.rstrip() for content in content_list]
-            return content_list
-        else:
-            content_list = [content.rstrip().split(split) for content in content_list]
-        if convertNum:
-            for i, line in enumerate(content_list):
-                line_data = []
-                for l in line:
-                    if is_int(l):  # isdigit() 方法检测字符串是否只由数字组成,只能判断整数
-                        line_data.append(int(l))
-                    elif is_float(l):  # 判断是否为小数
-                        line_data.append(float(l))
-                    else:
-                        line_data.append(l)
-                content_list[i] = line_data
-    return content_list
+        lines = f.readlines()
+    lines = [line.strip().strip('\ufeff').strip('\xef\xbb\xbf') for line in lines]
+    if split is None:
+        return lines
+    else:
+        lines = [line.split(split) for line in lines]
+    if convertNum:
+        for i, line in enumerate(lines):
+            line = [str2number(l) for l in line]
+            lines[i] = line
+    return lines
+
+
+def str2number(x):
+    """
+    :param x:
+    :return:
+    """
+    # 如果已经是数值类型，直接处理
+    if isinstance(x, (int, float)):
+        return int(x) if isinstance(x, int) or x.is_integer() else x
+    # 处理字符串类型
+    if isinstance(x, str):
+        if not x: return x  # 处理空字符串
+        x = x.strip()  # 去除首尾空格
+        try:
+            x = int(x)
+        except Exception as e:
+            try:
+                x = float(x)
+            except Exception as e:
+                pass
+    return x
 
 
 def read_line_image_label(line_image_label):
-    '''
-    line_image_label:[image_id,boxes_nums,x1, y1, w, h, label_id,x1, y1, w, h, label_id,...]
+    """
+    line_image_label:[image_ids,boxes_nums,x1, y1, w, h, label_id,x1, y1, w, h, label_id,...]
     :param line_image_label:
     :return:
-    '''
+    """
     line_image_label = line_image_label.strip().split()
     image_id = line_image_label[0]
     boxes_nums = int(line_image_label[1])
@@ -245,6 +300,32 @@ def read_lines_image_labels(filename):
     return boxes_label_lists
 
 
+def split_letters_and_numbers(string, join=True):
+    """
+    切分字母和数字
+    :param string:
+    :param join:
+    :return:
+    """
+    letters = []
+    numbers = []
+    for char in string:
+        if char.isalpha():  # 判断字符是否是字母
+            letters.append(char)
+        elif char.isdigit():  # 判断字符是否是数字
+            numbers.append(char)
+    if join:
+        letters = "".join(letters)
+        numbers = "".join(numbers)
+    return letters, numbers
+
+
+def is_number(value):
+    if re.match(r'^[-+]?[0-9]+(\.[0-9]+)?$', value):
+        return True
+    return False
+
+
 def is_int(str):
     """
     判断是否为整数
@@ -297,6 +378,18 @@ def get_basename(file_list):
     return dest_list
 
 
+def change_postfix(file: str, postfix: str):
+    """
+    修改后缀名称
+    :param file:
+    :param postfix:
+    :return:
+    """
+    src_pos = file.split(".")[-1]
+    file = file[:-len(f".{src_pos}")] + postfix
+    return file
+
+
 def randam_select_images(image_list, nums, shuffle=True):
     """
     randam select nums images
@@ -325,6 +418,19 @@ def remove_dir(dir):
         shutil.rmtree(dir)
 
 
+def get_config_file(file_dir, prefix="*.yaml"):
+    """
+    获得config.yaml文件
+    :param file_dir:
+    :param prefix:
+    :return:
+    """
+    if os.path.isfile(file_dir): file_dir = os.path.dirname(file_dir)
+    files = get_prefix_files(file_dir, prefix)
+    file = files[0] if len(files) > 0 else ""
+    return file
+
+
 def get_prefix_files(file_dir, prefix):
     """
     获得符合前缀条件所有文件
@@ -444,13 +550,21 @@ def copy_dir_delete(src, dst):
     # time.sleep(3 / 1000.)
 
 
-def copy_dir(src, dst, sub=False):
+def copy_dir(src, dst, sub=False, exclude=[]):
     """ copy src-directory to dst-directory, will cover the same files"""
     if not os.path.exists(src):
         print("\nno src path:{}".format(src))
         return
     if sub: dst = os.path.join(dst, os.path.basename(src))
     for root, dirs, files in os.walk(src, topdown=False):
+        isExclude = False
+        if exclude:
+            for p2 in exclude:
+                p2 = p2[2:] if p2.startswith("./") else p2
+                p1 = root[len(src) + 1:]
+                isExclude = p1.startswith(p2)
+                if isExclude: break
+        if isExclude: continue
         dest_path = os.path.join(dst, os.path.relpath(root, src))
         if not os.path.exists(dest_path):
             os.makedirs(dest_path)
@@ -461,17 +575,34 @@ def copy_dir(src, dst, sub=False):
             )
 
 
+def move_dir(src, dst, sub=False):
+    """ copy src-directory to dst-directory, will cover the same files"""
+    if not os.path.exists(src):
+        print("\nno src path:{}".format(src))
+        return
+    if sub: dst = os.path.join(dst, os.path.basename(src))
+    for root, dirs, files in os.walk(src, topdown=False):
+        dest_path = os.path.join(dst, os.path.relpath(root, src))
+        if not os.path.exists(dest_path):
+            os.makedirs(dest_path)
+        for filename in files:
+            move_file(
+                os.path.join(root, filename),
+                os.path.join(dest_path, filename)
+            )
+
+
 def move_file(srcfile, dstfile):
     """ 移动文件或重命名"""
-    if not os.path.isfile(srcfile):
-        print("%s not exist!" % (srcfile))
-    else:
+    if os.path.exists(srcfile) and os.path.isfile(srcfile):
         fpath, fname = os.path.split(dstfile)  # 分离文件名和路径
         if not os.path.exists(fpath):
             os.makedirs(fpath)  # 创建路径
         shutil.move(srcfile, dstfile)
         # print("copy %s -> %s"%( srcfile,dstfile))
         # time.sleep(1 / 1000.)
+    else:
+        print("%s not exist!" % (srcfile))
 
 
 def copy_file(srcfile, dstfile):
@@ -515,6 +646,14 @@ def move_file_to_dir(srcfile, des_dir):
         move_file(srcfile, dstfile)  # 复制文件
 
 
+def copy_file_list(file_list, dst_dir):
+    [copy_file_to_dir(file, dst_dir) for file in file_list]
+
+
+def move_file_list(file_list, dst_dir):
+    [move_file_to_dir(file, dst_dir) for file in file_list]
+
+
 def merge_dir(src, dst, sub, merge_same):
     src_dir = os.path.join(src, sub)
     dst_dir = os.path.join(dst, sub)
@@ -542,6 +681,7 @@ def create_dir(parent_dir, dir1=None, filename=None):
     out_path = parent_dir
     if dir1:
         out_path = os.path.join(parent_dir, dir1)
+    if not out_path: return out_path
     if not os.path.exists(out_path):
         os.makedirs(out_path)
     if filename:
@@ -562,11 +702,11 @@ def create_file_path(filename):
 
 
 def get_sub_paths(input_dir):
-    '''
+    """
     当前路径下所有子目录
     :param input_dir:
     :return:
-    '''
+    """
     sub_list = []
     for root, dirs, files in os.walk(input_dir):
         sub_list = dirs
@@ -593,6 +733,43 @@ def get_sub_list(file_list, dirname: str):
     return sub_list
 
 
+def get_train_test_files(file_dir, ratio=0.2, postfix=IMG_POSTFIX, subname="", shuffle=False, sub=False, save=True):
+    """
+    划分训练集和测试集
+    :param file_dir:
+    :param ratio: 若小于0，则表示test/train的比例；若大于0，则表示test的样本数，剩下的为train数目
+    :param postfix:
+    :param subname:
+    :param shuffle:
+    :param sub:
+    :return:
+    """
+    file_list = get_files_lists(file_dir, postfix=postfix, subname=subname,
+                                shuffle=shuffle, sub=sub)
+    if shuffle:
+        random.seed(100)
+        random.shuffle(file_list)
+    nums = len(file_list)
+    test_nums = ratio if ratio > 1 else int(len(file_list) * ratio)
+    test, train = file_list[0:test_nums], file_list[test_nums:]
+    train.sort()
+    test.sort()
+    if os.path.isdir(file_dir):
+        out = file_dir
+    elif os.path.isfile(file_dir):
+        out = os.path.dirname(file_dir)
+    else:
+        out = file_dir
+    if save:
+        write_list_data(os.path.join(out, f"total-{nums}.txt"), file_list)
+        write_list_data(os.path.join(out, f"train-{len(train)}.txt"), train)
+        write_list_data(os.path.join(out, f"test-{len(test)}.txt"), test)
+    print("total files: {}".format(nums))
+    print("train files: {}".format(len(train)))
+    print("test  files: {}".format(len(test)))
+    return train, test
+
+
 def get_all_files(file_dir):
     """获取file_dir目录下，所有文本路径，包括子目录文件"""
     file_list = []
@@ -603,38 +780,42 @@ def get_all_files(file_dir):
     return file_list
 
 
-def get_files_lists(file_dir, postfix=IMG_POSTFIX, subname="", shuffle=False):
+def get_files_lists(file_dir, postfix=IMG_POSTFIX, subname="", shuffle=False, sub=False):
     """
     读取文件和列表: list,*.txt ,image path, directory
     :param file_dir: list,*.txt ,image path, directory
     :param subname: "JPEGImages"
+    :param sub: 是否去除根路径
     :return:
     """
     if isinstance(file_dir, list):
-        image_list = file_dir
+        file_list = file_dir
     elif file_dir.endswith(".txt"):
         data_root = os.path.dirname(file_dir)
-        image_list = read_data(file_dir, split=None)
-        if subname: image_list = [os.path.join(data_root, subname, n) for n in image_list]
+        file_list = read_data(file_dir, split=None)
+        if subname: file_list = [os.path.join(data_root, subname, n) for n in file_list]
     elif os.path.isdir(file_dir):
-        image_list = get_files_list(file_dir, prefix="", postfix=postfix)
+        file_list = get_files_list(file_dir, prefix="", postfix=postfix)
     elif os.path.isfile(file_dir):
-        image_list = [file_dir]
+        file_list = [file_dir]
     else:
-        raise Exception("Error:{}".format(file_dir))
+        file_list = [file_dir]
+        # raise Exception("Error:{}".format(file_dir))
     if shuffle:
         random.seed(100)
-        random.shuffle(image_list)
-    return image_list
+        random.shuffle(file_list)
+    if sub: file_list = get_sub_list(file_list, dirname=file_dir)
+    return file_list
 
 
-def get_files_list(file_dir, prefix="", postfix=None, basename=False):
+def get_files_list_v1(file_dir, prefix="", postfix=None, basename=False, sub=False):
     """
     获得file_dir目录下，后缀名为postfix所有文件列表，包括子目录所有文件
     :param file_dir:
     :param prefix: 前缀
     :param postfix: 后缀
     :param basename: 返回的列表是文件名（True），还是文件的完整路径(False)
+    :param sub: 是否去除根路径
     :return:
     """
     file_list = []
@@ -652,6 +833,26 @@ def get_files_list(file_dir, prefix="", postfix=None, basename=False):
                 file_list.append(file)
     file_list.sort()
     file_list = get_basename(file_list) if basename else file_list
+    if sub: file_list = get_sub_list(file_list, dirname=file_dir)
+    return file_list
+
+
+def get_files_list(file_dir, prefix="", postfix=None, basename=False, sub=False):
+    """
+    获得file_dir目录下，后缀名为postfix所有文件列表，包括子目录所有文件
+    :param file_dir:
+    :param prefix: 前缀
+    :param postfix: 后缀
+    :param basename: 返回的列表是文件名（True），还是文件的完整路径(False)
+    :param sub: 是否去除根路径
+    :return:
+    """
+    file_list = get_all_files(file_dir)
+    file_list = text_utils.find_match_texts(file_list, pattern=[prefix], org=True) if prefix else file_list
+    file_list = text_utils.find_match_texts(file_list, pattern=postfix, org=True) if postfix else file_list
+    file_list.sort()
+    file_list = get_basename(file_list) if basename else file_list
+    if sub: file_list = get_sub_list(file_list, dirname=file_dir)
     return file_list
 
 
@@ -797,35 +998,105 @@ def print_dict(dict_data, save_path):
                 f.writelines(info + "\n")
 
 
+def get_pair_data(image_dir, pair_num=-1):
+    """
+    获得图片对数据
+    :param image_dir:
+    :param pair_num:-1 表示所有对
+    :return:
+    """
+    max_nums = int(pair_num / 2)
+    image_list = get_files_lists(image_dir)
+    image_list = get_sub_list(image_list, dirname=image_dir)
+    nums = len(image_list)
+    print("have {} images and {} combinations".format(nums, nums * (nums - 1) / 2))
+    pairs = []
+    for paths in itertools.combinations(image_list, 2):
+        file1, file2 = paths
+        label1 = file1.split(os.sep)[0]
+        label2 = file2.split(os.sep)[0]
+        if label1 == label2:
+            pairs.append([file1, file2, 1])
+        else:
+            pairs.append([file1, file2, 0])
+    pairs = np.asarray(pairs)
+    pairs = pairs[np.lexsort(pairs.T)]
+    pair0 = pairs[pairs[:, -1] == "0", :]
+    pair1 = pairs[pairs[:, -1] == "1", :]
+    nums1 = len(pair1)
+    nums0 = len(pair0)
+    if pair_num < 0: max_nums = nums1
+    if max_nums > nums1:
+        raise Exception("pair_nums({}) must be less than num_pair1({})".format(max_nums, nums1))
+    index_0 = np.random.permutation(nums0)[:max_nums]  # 打乱后的行号
+    index_1 = np.random.permutation(nums1)[:max_nums]  # 打乱后的行号
+    pair0 = pair0[index_0, :]  # 获取打乱后的训练数据
+    pair1 = pair1[index_1, :]  # 获取打乱后的训练数据
+    pairs = np.concatenate([pair0, pair1], axis=0).tolist()
+    print("have {} pairs，pair0 nums:{}，pair1 nums:{}".format(len(pairs), len(pair0), len(pair1)))
+    return pairs
+
+
+def get_pair_files(data_root, out_root=None, image_sub="", label_sub="",
+                   postfix=IMG_POSTFIX, label_postfix="txt", shuffle=False):
+    """
+    获得同目录下一对文件
+    :param data_root: 根目录
+    :param out_root:
+    :param image_sub:
+    :param label_sub:
+    :param label_postfix: label文件后缀，如txt,png,json等
+    :return:
+    """
+    image_dir = os.path.join(data_root, image_sub)
+    label_dir = os.path.join(data_root, label_sub)
+    if out_root: create_dir(out_root)
+    file_list = get_files_lists(file_dir=data_root, postfix=postfix, subname="", shuffle=shuffle, sub=False)
+    pair_list = []
+    for i, image_name in tqdm(enumerate(file_list)):
+        postfix = image_name.split(".")[-1]
+        lable_name = image_name.replace(f".{postfix}", f".{label_postfix}")
+        image_file = os.path.join(image_dir, image_name)
+        lable_file = os.path.join(label_dir, lable_name)
+        if os.path.exists(image_file) and os.path.exists(lable_file):
+            image_file, lable_file = get_sub_list([image_file, lable_file], dirname=data_root)
+            pair_list.append([image_file, lable_file])
+    if out_root:
+        filename = os.path.join(out_root, "file_list.txt")
+        write_data(filename, pair_list, split=",", mode='w')
+    return pair_list
+
+
 def read_pair_data(filename, split=True):
-    '''
+    """
     read pair data,data:[image1.jpg image2.jpg 0]
     :param filename:
     :param split:
     :return:
-    '''
-    content_list = read_data(filename)
+    """
+    pair_list = read_data(filename)
     if split:
-        content_list = np.asarray(content_list)
-        faces_list1 = content_list[:, :1].reshape(-1)
-        faces_list2 = content_list[:, 1:2].reshape(-1)
+        pair_list = np.asarray(pair_list)
+        pair1 = pair_list[:, :1].reshape(-1)
+        pair2 = pair_list[:, 1:2].reshape(-1)
         # convert to 0/1
-        issames_data = np.asarray(content_list[:, 2:3].reshape(-1), dtype=np.int)
+        issames_data = np.asarray(pair_list[:, 2:3].reshape(-1), dtype=np.int)
         issames_data = np.where(issames_data > 0, 1, 0)
-        faces_list1 = faces_list1.tolist()
-        faces_list2 = faces_list2.tolist()
+        pair1 = pair1.tolist()
+        pair2 = pair2.tolist()
         issames_data = issames_data.tolist()
-        return faces_list1, faces_list2, issames_data
-    return content_list
+        return pair1, pair2, issames_data
+    return pair_list
 
 
 def check_files(files_list, sizeTh=1 * 1024, isRemove=False):
-    ''' 去除不存的文件和文件过小的文件列表
+    """
+    去除不存的文件和文件过小的文件列表
     :param files_list:
     :param sizeTh: 文件大小阈值,单位：字节B，默认1000B ,33049513/1024/1024=33.0MB
     :param isRemove: 是否在硬盘上删除被损坏的原文件
     :return:
-    '''
+    """
     i = 0
     while i < len(files_list):
         path = files_list[i]
@@ -871,7 +1142,7 @@ def get_loacl_eth2():
     eth_list = []
     os.system("ls -l /sys/class/net/ | grep -v virtual | sed '1d' | awk 'BEGIN {FS=\"/\"} {print $NF}' > eth.yaml")
     try:
-        with open('./eth.yaml', "r") as f:
+        with open('./eth.yaml', "rb") as f:
             for line in f.readlines():
                 line = line.strip()
                 eth_list.append(line.lower())
@@ -974,6 +1245,134 @@ def load_pickle(file):
     return obj
 
 
+def copy_move_file_dir(src, dst, postfix=None, sub_names=None, max_nums=None, shuffle=True, move=False):
+    """
+    复制/移动文件夹
+    :param src: 原始目录
+    :param dst: 输出目录
+    :param sub_names: 需要复制/移动的子文件夹名称，默认全部
+    :param max_nums: 最大移动数量，默认不限制
+    :param shuffle:
+    :param move: True表示移动，False表示复制
+    :return: out_list 返回复制/移动后，out_dir的文件列表
+    """
+    file_list = get_files_list(src, prefix="", postfix=postfix, basename=False)
+    file_list = get_sub_list(file_list, src)
+    if shuffle:
+        random.seed(100)
+        random.shuffle(file_list)
+        random.shuffle(file_list)
+    if max_nums: file_list = file_list[:min(max_nums, len(file_list))]
+    for file_name in tqdm(file_list):
+        sub = os.path.dirname(file_name)
+        if sub_names and (sub not in sub_names): continue
+        src_file = os.path.join(src, file_name)
+        out_file = os.path.join(dst, file_name)
+        if move:
+            move_file(src_file, out_file)
+        else:
+            copy_file(src_file, out_file)
+    out_list = get_files_list(dst, prefix="", postfix=postfix, basename=False)
+    return out_list
+
+
+def copy_move_dir_dir(src, dst, postfix=None, sub_names=None, per_nums=None, shuffle=True, move=False):
+    """
+    复制/移动文件夹
+    :param src: 原始目录
+    :param out_dir_: 输出目录
+    :param sub_names: 需要复制/移动的子文件夹名称，默认全部
+    :param per_nums: 每个子文件夹最大移动数量，默认全部
+    :param shuffle:
+    :param move: True表示移动，False表示复制
+    :return: out_list 返回复制/移动后，out_dir的文件列表
+    """
+    sub_list = get_sub_paths(src)
+    for sub in tqdm(sub_list):
+        if sub_names and (sub not in sub_names): continue
+        src_dir_ = os.path.join(src, sub)
+        out_dir_ = os.path.join(dst, sub)
+        if per_nums is None:
+            if move:
+                move_dir(src_dir_, out_dir_)
+            else:
+                copy_dir(src_dir_, out_dir_)
+        else:
+            copy_move_file_dir(src_dir_, out_dir_, postfix=postfix, sub_names=postfix,
+                               max_nums=per_nums, shuffle=shuffle, move=move)
+    out_list = get_files_list(dst, prefix="", postfix=postfix, basename=False)
+    return out_list
+
+
+def get_voc_file_list(voc_root,
+                      image_dir="",
+                      annos_dir="",
+                      prefix="",
+                      postfix=IMG_POSTFIX,
+                      only_id=False,
+                      check=True,
+                      shuffle=False,
+                      max_num=None):
+    """
+    获得VOC数据集的文件列表，并剔除无效的文件
+    :param voc_root:
+    :param prefix:
+    :param postfix:
+    :param only_id:
+    :param check: 检测xml和图片是否存在
+    :param shuffle:
+    :param max_num:
+    :return:
+    """
+    image_dir = image_dir if image_dir else os.path.join(voc_root, "JPEGImages")
+    annos_dir = annos_dir if annos_dir else os.path.join(voc_root, "Annotations")
+    filename = os.path.join(os.path.dirname(image_dir), "file_list.txt")
+    file_list = get_files_list(image_dir, prefix=prefix, postfix=postfix, basename=False)
+    file_list = get_sub_list(file_list, dirname=image_dir)
+    if check:
+        xmls_list = get_files_list(annos_dir, postfix=["*.xml"], basename=True)
+        print("xml file:{},image file:{}".format(len(xmls_list), len(file_list)))
+        xmls_ids = [str(f).split(".")[0] for f in xmls_list]
+        file_list = [f for f in file_list if str(f).split(".")[0] in xmls_ids]
+    if only_id:
+        file_list = [str(f).split(".")[0] for f in file_list]
+    if shuffle:
+        random.seed(100)
+        random.shuffle(file_list)
+    if max_num:
+        max_num = min(max_num, len(file_list))
+        file_list = file_list[0:max_num]
+    write_list_data(filename, file_list)
+    print("num files:{},out_path:{}".format(len(file_list), filename))
+
+
+def copy_move_voc_dataset(data_file, data_root=None, out_root=None, file_map={}, move=False):
+    """
+    :param data_file: voc file.txt
+    :param data_root: voc dataset root path
+    :param out_root:  new output root path
+    :param file_map: {"Annotations": "xml", "json": "json", "JPEGImages": None}
+    :param move: move or copy
+    :return:
+    """
+    if not data_root: data_root = os.path.dirname(data_file)
+    files = read_data(data_file, split=None)
+    for name in tqdm(files):
+        idx, pos = name.split(".")
+        for sub, p in file_map.items():
+            n = name.replace(f".{pos}", f".{p}") if p else name
+            path = os.path.join(data_root, sub, n)
+            if out_root and os.path.exists(path):
+                dest = os.path.join(out_root, sub, n)
+                if move:
+                    move_file(path, dest)
+                else:
+                    copy_file(path, dest)
+            else:
+                print(f"no file:{path}")
+    return files
+
+
 if __name__ == '__main__':
     dir = "/home/dm/nasdata/dataset-dmai/handwriting/word-class/trainval/unknown"
     file_list, label_list = get_files_labels(dir)
