@@ -1,7 +1,7 @@
-# -*-coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 """
-    @Author : panjq
-    @E-mail : pan_jinquan@163.com
+    @Author : Pan
+    @E-mail : 390737991@qq.com
     @Date   : 2022-04-19 10:40:26
     @Brief  :
 """
@@ -20,6 +20,17 @@ class Dict2Obj(object):
         self.__dict__.update(args)
 
 
+def dict_sort_by_value(data: Dict, reverse=False):
+    """
+    按照字典的value值排序
+    :param src:
+    :param reverse: False 升序
+                    True  降序
+    """
+    dst = dict(sorted(data.items(), key=lambda x: x[1], reverse=reverse))
+    return dst
+
+
 def formatting(content):
     """格式化json数据"""
     info = json.dumps(content, indent=1, separators=(', ', ': '), ensure_ascii=False)
@@ -79,6 +90,13 @@ def set_values(content, keys, values):
     return content
 
 
+def set_value(content, key, value):
+    """根据keys路径设置对应的值"""
+    # content = toolz_assoc_in(content, keys=key, value=value)
+    content = toolz.assoc_in(content, keys=key, value=value)
+    return content
+
+
 def toolz_assoc_in(content, keys, value):
     """toolz_assoc_in用来代替toolz.assoc_in"""
     cur_keys = []
@@ -119,7 +137,7 @@ if __name__ == "__main__":
     # toolz使用toolz工具或得所有keys的值,values1与values的值是一样的
     # values1 = get_values(content, keys=keys)
     # values1 = get_values(content, keys=[['data1', 'image', 1], ['data', 'file', 'file11']])
-    values1 = get_value(None, key=['data', 'image', 1],default={"data"})
+    values1 = get_value(None, key=['data', 'image', 1], default={"data"})
     print(values1)
     print("===" * 20)
     values = list(range(len(values)))
