@@ -1,9 +1,8 @@
-# -*-coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 """
-    @Project: python-learning-notes
     @File   : numpy_tools.py
-    @Author : panjq
-    @E-mail : pan_jinquan@163.com
+    @Author : Pan
+    @E-mail : 390737991@qq.com
     @Date   : 2019-10-23 12:01:36
 """
 import numpy as np
@@ -12,30 +11,104 @@ from sklearn import metrics, preprocessing
 import heapq
 
 
-def matching_embedding(inputs, target, use_max=False):
+def softmax(x, axis=1):
+    # 计算每行的最大值
+    row_max = x.max(axis=axis)
+    # 每行元素都需要减去对应的最大值，否则求exp(x)会溢出，导致inf情况
+    # row_max = row_max.reshape(-1, 1)
+    row_max = row_max[:, :, np.newaxis]
+    x = x - row_max
+    # 计算e的指数次幂
+    x_exp = np.exp(x)
+    x_sum = np.sum(x_exp, axis=axis, keepdims=True)
+    s = x_exp / x_sum
+    return s
+
+def sigmoid(x):
+    return 1 / (1 + np.exp(-x))
+
+
+def gaussian_impulse(x, c=0, sigma=0.1):
     """
-    按照最小L2距离进行特征匹配
+    高斯脉冲
+    :param x: 输入数据x
+    :param c: 脉冲中心点
+    :param sigma:标准差，越小脉冲越尖锐
+    :return: 生成高斯脉冲
+    """
+    y = np.exp(-((x - c) ** 2) / (2 * sigma ** 2))
+    return y
+
+
+def power(x, n=1):
+    """幂函数"""
+    y = np.power(x, n)
+    return y
+
+
+def feature_norm(x, axis=-1):
+    """
+    特征归一化,会使得特征L2之和为1, 即||y||=L2(y,axis=-1)
+    y = x / x.norm(dim=-1, keepdim=True)               # torch
+    y = x / np.linalg.norm(x, axis=-1, keepdims=True)  # numpy
+    :param x: 输入二维矩阵(N,embedding-size)，每行是一个样本，样本特征维度为embedding-size
+    :param axis:
+    :return:
+    """
+    y = x / np.linalg.norm(x, axis=axis, keepdims=True)
+    return y
+
+
+def torch_norm(x, axis=-1):
+    """
+    特征归一化:
+    y = x / x.norm(dim=-1, keepdim=True)               # torch
+    y = x / np.linalg.norm(x, axis=-1, keepdims=True)  # numpy
+    :param x: 输入二维矩阵(N,embedding-size)，每行是一个样本，样本特征维度为embedding-size
+    :param axis:
+    :return:
+    """
+    import torch
+    x = torch.from_numpy(x)
+    y = x / x.norm(dim=-1, keepdim=True)  # torch
+    return y
+
+
+def feature_similarity(inputs, target):
+    """
+    计算特征相似性
+    similarity = inputs @ target.T                 # torch
+    similarity = np.sum(inputs * target, axis=-1)  # numpy
     :param inputs: 输入待匹配的特点，shape=(n,D),其中n表示样本个数，D表示特征维度
     :param target: 目标匹配数据库,shape=(N,D),其中N表示数据库样本个数，D表示特征维度
-    :param use_max: False:按照最小距离进行匹配,True:按照最大距离进行匹配
-    :return: indexes： 与target匹配的索引
-             distance：与target匹配的最小L2距离(欧式距离=np.sqrt(L2)=np.sqrt(distance))
+    :return: similarity
     """
     assert inputs.shape[1] == target.shape[1]  # 特征维度必须一致
-    data = np.expand_dims(inputs, axis=-1)  # (n,512)->(n,512,1)
-    dataset = target.image_demo(1, 0)  # (N, 512)->(512, N)
-    dataset = np.expand_dims(dataset, axis=0)  # (1,512, N)
-    diff = data - dataset  # (n, 512, 1) - (1, 512, N) = (n, 512, N)
-    dist = np.sum(np.power(diff, 2), axis=1)  # (n, N)
+    inputs_ = np.expand_dims(inputs, axis=1)  # (b,512)->(b,1,512)
+    target_ = np.expand_dims(target, axis=0)  # (n, 512)->(1, n, 512)
+    similarity = np.sum(inputs_ * target_, axis=-1)  # (b,1,512) * (1, n, 512)
+    return similarity
+
+
+def feature_matching(inputs, target, use_max=True):
+    """
+    按照相似程度进行特征匹配，输入特征数据必须进行特征归一化:
+    y = x / x.norm(dim=-1, keepdim=True)               # torch
+    y = x / np.linalg.norm(x, axis=-1, keepdims=True)  # numpy
+    :param inputs: 输入待匹配的特点，shape=(n,D),其中n表示样本个数，D表示特征维度
+    :param target: 目标匹配数据库,shape=(N,D),其中N表示数据库样本个数，D表示特征维度
+    :param use_max:按照相似程度进行匹配
+    :return: index： 与target匹配的索引
+             score：与target匹配的最小L2距离(欧式距离=np.sqrt(L2)=np.sqrt(distance))
+    """
+    similarity = feature_similarity(inputs, target)
     if use_max:
-        indexes = np.argmax(dist, axis=1)
-        distance = np.max(dist, axis=1)
-        # distance = np.asarray([dist[i][c] for i, c in enumerate(indexes)])
+        index = np.argmax(similarity, axis=1)
+        score = np.max(similarity, axis=1)
     else:
-        indexes = np.argmin(dist, axis=1)
-        distance = np.min(dist, axis=1)
-        # distance = np.asarray([dist[i][c] for i, c in enumerate(indexes)])
-    return indexes, distance
+        index = np.argmin(similarity, axis=1)
+        score = np.min(similarity, axis=1)
+    return index, score
 
 
 def get_nearest_point(points, center, axis=1, use_max=False):
@@ -297,8 +370,31 @@ def __print(data, info=""):
         print("{}".format(data[index]))
 
 
+def euclidean_distance(p1, p2, axis=1):
+    """
+    计算欧氏距离
+    point1 = [[3, 4], [4, 3], [4, 3]]
+    center = [[0, 0]]
+    point1 = np.asarray(point1)
+    center = np.asarray(center)
+
+    L2-norm=np.sqrt(np.sum(np.square(x), axis=axis))
+    L1-norm=np.sum(np.abs(x), axis=axis))
+    # 下面三个计算是等价的
+    d1 = np.sqrt(numpy_tools.l2(point1 - center, axis=1)) # L2开根号就是欧式距离
+    d2 = numpy_utils.norm(point1 - center, p=2, axis=1)
+    d3 = numpy_utils.euclidean_distance(point1, center, axis=1)
+    """
+    d = np.sqrt(np.sum(np.square(p1 - p2), axis=axis))
+    return d
+
+
 def norm(x, p=1, axis=0):
-    """L-p范数"""
+    """
+    L-p范数
+    L2-norm=np.sqrt(np.sum(np.square(x), axis=axis))
+    L1-norm=np.sum(np.abs(x), axis=axis))
+    """
     y = np.linalg.norm(x, ord=p, axis=axis, keepdims=True)
     return y
 
@@ -309,8 +405,14 @@ def l2(x, axis=0):
     return y
 
 
+def l2_norm(x, axis=0):
+    """L-2范数"""
+    y = np.sqrt(l2(x, axis=axis))
+    return y
+
+
 def l1(x, axis=0):
-    """L-1范数"""
+    """L-1范数:"""
     y = np.sum(np.abs(x), axis=axis)
     return y
 
@@ -335,22 +437,6 @@ def mean_absolute_error(y_true, y_pre):
     return l1 / y_true.size
 
 
-def euclidean_distance(p1, p2, axis=1):
-    """
-    计算欧氏距离
-    point1 = [[3, 4], [4, 3], [4, 3]]
-    center = [[0, 0]]
-    point1 = np.asarray(point1)
-    center = np.asarray(center)
-    # 下面三个计算是等价的
-    d1 = np.sqrt(numpy_tools.l2(point1 - center, axis=1)) # L2开根号就是欧式距离
-    d2 = numpy_utils.norm(point1 - center, p=2, axis=1)
-    d3 = numpy_utils.euclidean_distance(point1, center, axis=1)
-    """
-    d = np.sqrt(np.sum(np.square(p1 - p2), axis=axis))
-    return d
-
-
 def mean(data):
     return np.mean(data)
 
@@ -502,37 +588,29 @@ def get_error(data1, data2):
     return mse, rmse, mae
 
 
-def get_topK(data, k=1, axis=-1, reverse=False):
+def get_topK(data: np.ndarray, K: int, reverse=False):
     """
-    选取Top5: contours = sorted(contours, key=cv2.contourArea, reverse=True)[:5]
-    多维数组排序
-    Args:
-        data: 多维数组
-        k: 取数
-        axis: 轴维度
-        reverse: 是否倒序
-    Returns:
-        top_sorted_scores: 值
-        top_sorted_indexes: 位置
+    计算data中前K大或前K小的元素
+    :param data: 输入数组
+    :param K: 要获取的元素个数
+    :param reverse: False表示获取前K大元素，True表示获取前K小元素
+    :return: (top_value, top_index) 值和对应的索引
     """
-    data = np.asarray(data)
+    K = min(len(data), K)
     if reverse:
-        partition_index = np.take(np.argpartition(data, kth=k, axis=axis), range(0, k), axis)
+        # 获取前K小的元素的索引
+        index = np.argpartition(data, K - 1)[:K]
+        # 按照值的大小对索引进行排序（从小到大）
+        top_index = index[np.argsort(data[index])]
     else:
-        # argpartition分区排序，在给定轴上找到最小的值对应的idx，partition同理找对应的值
-        # kth表示在前的较小值的个数，带来的问题是排序后的结果两个分区间是仍然是无序的
-        # kth绝对值越小，分区排序效果越明显
-        axis_length = data.shape[axis]
-        partition_index = np.take(np.argpartition(data, kth=-k, axis=axis),
-                                  range(axis_length - k, axis_length), axis)
-    top_scores = np.take_along_axis(data, partition_index, axis)
-    # 分区后重新排序
-    sorted_index = np.argsort(top_scores, axis=axis)
-    if not reverse:
-        sorted_index = np.flip(sorted_index, axis=axis)
-    top_k_value = np.take_along_axis(top_scores, sorted_index, axis)
-    top_k_index = np.take_along_axis(partition_index, sorted_index, axis)
-    return top_k_value, top_k_index
+        # 获取前K大的元素的索引
+        index = np.argpartition(data, -K)[-K:]
+        # 按照值的大小对索引进行排序（从大到小）
+        top_index = index[np.argsort(data[index])][::-1]
+
+    # 获取对应的值
+    top_value = data[top_index]
+    return top_value, top_index
 
 
 class Preprocessing(object):
@@ -609,19 +687,11 @@ class Preprocessing(object):
 if __name__ == "__main__":
     from pybaseutils import numpy_utils
 
-    x = np.array([[1., -1., 2.],
-                  [2., 0., 0.],
-                  [0., 1., -1.]])
-    y = numpy_utils.Preprocessing.feature_norm(x)
-    print(y)
-    # y = Preprocessing.minmax_scaler(x)
-    # y = Preprocessing.normalization(x)
-
-    value_list = [
-        [1, 2, 3, 4, 5, 6],
-        [5, 6, 1, 2, 3, 4],
-    ]
-    print(value_list)
-    top_k_value, top_k_index = get_topK(value_list, k=3, reverse=False)
-    print(top_k_value)
-    print(top_k_index)
+    y = np.array([[0., 0.1, 0, 1], [0, 0.1, 1, 1], [0, 0.1, 2, 1]])
+    x = np.array([[0, 0.1, 1, 1], [0, 0.1, 2, 1], [0., 0.1, 0, 1]])
+    x1 = feature_norm(x)
+    x2 = torch_norm(x).numpy()
+    print(l2(x1, axis=-1))
+    print(l2(x2, axis=-1))
+    y = feature_norm(y)
+    print(feature_matching(x1, y))
