@@ -1,15 +1,26 @@
-# -*-coding: utf-8 -*-
+# -*- coding: utf-8 -*-
 """
-    @Author : panjq
-    @E-mail : pan_jinquan@163.com
+    @Author : Pan
+    @E-mail : 390737991@qq.com
     @Date   : 2022-05-25 18:19:34
     @Brief  :
 """
 import time
+from datetime import datetime
 import numpy as np
 from typing import Dict, List
 
 
+def date2stamp(date, format='%Y-%m-%d %H:%M:%S'):
+    """将日期格式转换为时间戳"""
+    return time.mktime(time.strptime(date, format))
+
+
+def stamp2date(stamp, format='%Y-%m-%d %H:%M:%S'):
+    """将时间戳转换为日期格式"""
+    return datetime.fromtimestamp(stamp).strftime(format)
+
+
 class Recorder(object):
     def __init__(self):
         self.content: Dict = {}
@@ -51,7 +62,10 @@ def performance(tag=""):
                                                                                            content["avg"],
                                                                                            content["total"],
                                                                                            content["count"])
-            print("{} call {} elapsed: {}".format(tag, func.__name__, elapsed))
+            if tag:
+                print("{:20s}{:20s} elapsed: {}".format(tag, func.__name__, elapsed))
+            else:
+                print("{:20s} elapsed: {}".format(func.__name__, elapsed))
             return result
 
         return wrapper
@@ -78,37 +92,38 @@ class Performance(object):
                                                                                        content["avg"],
                                                                                        content["total"],
                                                                                        content["count"])
-        print("{} elapsed: {}".format(self.tag, elapsed))
+        tag_ = f"{self.tag} " if self.tag else ""
+        print("{:20s}elapsed: {}\t".format(tag_, elapsed))
 
     def task(self):
         pass
 
 
-@performance("")
+@performance("test1")
 def targe_func1():
     time.sleep(1)
 
 
-@performance("")
+@performance("test111111")
 def targe_func2():
     time.sleep(0.5)
 
 
 def targe_func3():
-    with Performance("targe_func3") as p:
+    with Performance("test222") as p:
         time.sleep(1)
 
 
 def targe_func4():
-    with Performance("targe_func4") as p:
+    with Performance("test22222222") as p:
         time.sleep(0.5)
 
 
 def targe_func():
     targe_func1()
     targe_func2()
-    targe_func3()
-    targe_func4()
+    # targe_func3()
+    # targe_func4()
 
 
 if __name__ == '__main__':
